use std::sync::Mutex;

use serialport::SerialPort;

use crate::error::{Error, SResult};

/// Handler of the serial port
pub struct SerialManager {
    serial: Option<Mutex<Box<dyn SerialPort>>>,
}

impl SerialManager {
    pub fn new() -> Self {
        Self { serial: None }
    }

    /// Open the serial port with the given `port` device path and `baudrate`
    pub fn open(&mut self, port: &str, baudrate: u32) -> SResult<()> {
        let port = serialport::new(port, baudrate).open()?;
        self.serial.replace(Mutex::new(port));
        Ok(())
    }

    /// Close the serial port if it is open
    pub fn close(&mut self) -> SResult<()> {
        if self.serial.is_some() {
            self.serial.take();
            Ok(())
        } else {
            Err(Error::SerialNotOpen)
        }
    }

    /// Read `n` bytes from the serial port
    pub fn read_n_bytes(&self, n: usize) -> SResult<Vec<u8>> {
        let mut port = self
            .serial
            .as_ref()
            .ok_or(Error::SerialNotOpen)?
            .lock()
            .unwrap();
        let mut buf = vec![0; n];
        port.read_exact(&mut buf)?;
        Ok(buf)
    }

    /// Write `data` to the serial port
    pub fn write_bytes(&self, data: &[u8]) -> SResult<()> {
        let mut port = self
            .serial
            .as_ref()
            .ok_or(Error::SerialNotOpen)?
            .lock()
            .unwrap();
        port.write_all(data)?;
        Ok(())
    }
}
